@@ -11,7 +11,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
 """
 This module defines Circuit and the setting for circuit.
 """
@@ -19,13 +18,8 @@ This module defines Circuit and the setting for circuit.
 import warnings
 from functools import partial
 from typing import Callable
+
 from . import gate
-from .backends.numpy_backend import NumPyBackend
-from .backends.numba_backend import numba_backend_lazy
-from .backends.qasm_output_backend import QasmOutputBackend
-from .backends.ibmq_backend import ibmq_backend
-from .backends.sympy_backend import SympyBackend
-from .backends.qgate_backend import QgateBackend
 
 GATE_SET = {
     "i": gate.IGate,
@@ -62,9 +56,12 @@ GATE_SET = {
     "rxx": gate.RXXGate,
     "ryy": gate.RYYGate,
     "rzz": gate.RZZGate,
+    "zz": gate.ZZGate,
     "ccx": gate.ToffoliGate,
     "toffoli": gate.ToffoliGate,
+    "ccz": gate.CCZGate,
     "cswap": gate.CSwapGate,
+    "mat1": gate.Mat1Gate,
     "reset": gate.Reset,
     "measure": gate.Measurement,
     "m": gate.Measurement,
@@ -72,15 +69,6 @@ GATE_SET = {
 
 GLOBAL_MACROS = {}
 
-BACKENDS = {
-    "numpy": NumPyBackend,
-    "numba": numba_backend_lazy,
-    "qasm_output": QasmOutputBackend,
-    "ibmq": ibmq_backend,
-    "sympy_unitary": SympyBackend,
-    "qgate": QgateBackend,
-}
-DEFAULT_BACKEND_NAME = "numpy"
 
 class Circuit:
     """Store the gate operations and call the backends."""
@@ -91,7 +79,8 @@ class Circuit:
         self.n_qubits = n_qubits
 
     def __repr__(self):
-        return f'Circuit({self.n_qubits}).' + '.'.join(str(op) for op in self.ops)
+        return f'Circuit({self.n_qubits}).' + '.'.join(
+            str(op) for op in self.ops)
 
     def __get_backend(self, backend_name):
         try:
@@ -105,8 +94,10 @@ class Circuit:
 
     def __backend_runner_wrapper(self, backend_name):
         backend = self.__get_backend(backend_name)
+
         def runner(*args, **kwargs):
             return backend.run(self.ops, self.n_qubits, *args, **kwargs)
+
         return runner
 
     def __getattr__(self, name):
@@ -119,7 +110,8 @@ class Circuit:
             if backend_name in BACKENDS:
                 return self.__backend_runner_wrapper(backend_name)
             raise AttributeError(f"Backend '{backend_name}' is not exists.")
-        raise AttributeError(f"'circuit' object has no attribute or gate '{name}'")
+        raise AttributeError(
+            f"'circuit' object has no attribute or gate '{name}'")
 
     def __add__(self, other):
         if not isinstance(other, Circuit):
@@ -135,13 +127,16 @@ class Circuit:
         self.n_qubits = max(self.n_qubits, other.n_qubits)
         return self
 
-    def copy(self, copy_backends=True, copy_default_backend=True,
-             copy_cache=None, copy_history=None):
+    def copy(self,
+             copy_backends=True,
+             copy_default_backend=True,
+             copy_cache=None,
+             copy_history=None):
         """Copy the circuit.
 
-        :params
-        copy_backends :bool copy backends if True.
-        copy_default_backend :bool copy default_backend if True.
+        params:
+            | copy_backends :bool copy backends if True.
+            | copy_default_backend :bool copy default_backend if True.
         """
         copied = Circuit(self.n_qubits, self.ops.copy())
         if copy_backends:
@@ -151,20 +146,25 @@ class Circuit:
 
         # Warn for deprecated options
         if copy_cache is not None:
-            warnings.warn("copy_cache is deprecated. Use copy_backends instead.",
-                          DeprecationWarning)
+            warnings.warn(
+                "copy_cache is deprecated. Use copy_backends instead.",
+                DeprecationWarning)
         if copy_history is not None:
             warnings.warn("copy_history is deprecated.", DeprecationWarning)
 
         return copied
 
-    def dagger(self, ignore_measurement=False, copy_backends=False, copy_default_backend=True):
+    def dagger(self,
+               ignore_measurement=False,
+               copy_backends=False,
+               copy_default_backend=True):
         """Make Hermitian conjugate of the circuit.
 
         This feature is beta. Interface may be changed.
 
-        ignore_measurement (bool, optional): If True, ignore the measurement in the circuit.
-                                             Otherwise, if measurement in the circuit, raises ValueError.
+        ignore_measurement (bool, optional): 
+            | If True, ignore the measurement in the circuit.
+            | Otherwise, if measurement in the circuit, raises ValueError.
         """
         ops = []
         for g in reversed(self.ops):
@@ -172,8 +172,9 @@ class Circuit:
                 ops.append(g.dagger())
             except ValueError:
                 if not ignore_measurement:
-                    raise ValueError('Cannot make the Hermitian conjugate of this circuit because '
-                                     'the circuit contains measurement.')
+                    raise ValueError(
+                        'Cannot make the Hermitian conjugate of this circuit because '
+                        'the circuit contains measurement.')
 
         copied = Circuit(self.n_qubits, ops)
         if copy_backends:
@@ -194,12 +195,12 @@ class Circuit:
         However, following parameters are commonly used.
 
         Commonly used args (Depends on backend):
-            shots (int, optional): The number of sampling the circuit.
-            returns (str, optional):  The category of returns value.
-                e.g. "statevector" returns the state vector after run the circuit.
-                     "shots" returns the counter of measured value.
-            token, url (str, optional): The token and URL for cloud resource.
-
+            | shots (int, optional): The number of sampling the circuit.
+            | returns (str, optional):  The category of returns value.
+            |   e.g. "statevector" returns the state vector after run the circuit.
+            |         "shots" returns the counter of measured value.
+            | token, url (str, optional): The token and URL for cloud resource.
+        
         Returns:
             Depends on backend.
 
@@ -246,8 +247,8 @@ class Circuit:
         If you want to use global default setting, call this method with backend_name=None.
 
         Args:
-            backend_name (str or None): new default backend name.
-                If None is given, global setting is applied.
+            | backend_name (str or None): new default backend name.
+            |     If None is given, global setting is applied.
 
         Raises:
             ValueError: If `backend_name` is not registered backend.
@@ -281,9 +282,11 @@ class _GateWrapper:
 
     def __getitem__(self, args):
         self.target = args
-        self.circuit.ops.append(self.gate(self.target, *self.args, **self.kwargs))
+        self.circuit.ops.append(
+            self.gate(self.target, *self.args, **self.kwargs))
         # ad-hoc
-        self.circuit.n_qubits = max(gate.get_maximum_index(args) + 1, self.circuit.n_qubits)
+        self.circuit.n_qubits = max(
+            gate.get_maximum_index(args) + 1, self.circuit.n_qubits)
         return self.circuit
 
     def __str__(self):
@@ -297,17 +300,20 @@ class _GateWrapper:
             args_str = ""
         return self.name + args_str + " " + str(self.target)
 
+
 class BlueqatGlobalSetting:
     """Setting for Blueqat."""
     @staticmethod
-    def register_macro(name: str, func: Callable, allow_overwrite: bool = False) -> None:
+    def register_macro(name: str,
+                       func: Callable,
+                       allow_overwrite: bool = False) -> None:
         """Register new macro to Circuit.
 
         Args:
-            name (str): The name of macro.
-            func (callable): The function to be called.
-            allow_overwrite (bool, optional): If True, allow to overwrite the existing macro.
-                Otherwise, raise the ValueError.
+            | name (str): The name of macro.
+            | func (callable): The function to be called.
+            | allow_overwrite (bool, optional): If True, allow to overwrite the existing macro.
+            |     Otherwise, raise the ValueError.
 
         Raises:
             ValueError: The name is duplicated with existing macro, gate or method.
@@ -320,12 +326,15 @@ class BlueqatGlobalSetting:
                 raise ValueError(f"Circuit has attribute `{name}`.")
         if name.startswith("run_with_"):
             if allow_overwrite:
-                warnings.warn(f"Gate name `{name}` may conflict with run of backend.")
+                warnings.warn(
+                    f"Gate name `{name}` may conflict with run of backend.")
             else:
-                raise ValueError(f"Gate name `{name}` shall not start with 'run_with_'.")
+                raise ValueError(
+                    f"Gate name `{name}` shall not start with 'run_with_'.")
         if not allow_overwrite:
             if name in GATE_SET:
-                raise ValueError(f"Gate '{name}' is already exists in gate set.")
+                raise ValueError(
+                    f"Gate '{name}' is already exists in gate set.")
             if name in GLOBAL_MACROS:
                 raise ValueError(f"Macro '{name}' is already exists.")
         GLOBAL_MACROS[name] = func
@@ -349,10 +358,10 @@ class BlueqatGlobalSetting:
         """Register new gate to gate set.
 
         Args:
-            name (str): The name of gate.
-            gateclass (type): The type object of gate.
-            allow_overwrite (bool, optional): If True, allow to overwrite the existing gate.
-                Otherwise, raise the ValueError.
+            | name (str): The name of gate.
+            | gateclass (type): The type object of gate.
+            | allow_overwrite (bool, optional): If True, allow to overwrite the existing gate.
+            |     Otherwise, raise the ValueError.
 
         Raises:
             ValueError: The name is duplicated with existing gate.
@@ -365,12 +374,15 @@ class BlueqatGlobalSetting:
                 raise ValueError(f"Circuit has attribute `{name}`.")
         if name.startswith("run_with_"):
             if allow_overwrite:
-                warnings.warn(f"Gate name `{name}` may conflict with run of backend.")
+                warnings.warn(
+                    f"Gate name `{name}` may conflict with run of backend.")
             else:
-                raise ValueError(f"Gate name `{name}` shall not start with 'run_with_'.")
+                raise ValueError(
+                    f"Gate name `{name}` shall not start with 'run_with_'.")
         if not allow_overwrite:
             if name in GATE_SET:
-                raise ValueError(f"Gate '{name}' is already exists in gate set.")
+                raise ValueError(
+                    f"Gate '{name}' is already exists in gate set.")
             if name in GLOBAL_MACROS:
                 raise ValueError(f"Macro '{name}' is already exists.")
         GATE_SET[name] = gateclass
@@ -394,9 +406,10 @@ class BlueqatGlobalSetting:
         """Register new backend.
 
         Args:
-            name (str): The name of backend.
-            gateclass (type): The type object of backend
-            allow_overwrite (bool, optional): If True, allow to overwrite the existing backend.
+            | name (str): The name of backend.
+            | gateclass (type): The type object of backend
+            | allow_overwrite (bool, optional): If True, allow to overwrite the existing backend.
+              
                 Otherwise, raise the ValueError.
 
         Raises:
@@ -410,7 +423,8 @@ class BlueqatGlobalSetting:
                 raise ValueError(f"Circuit has attribute `run_with_{name}`.")
         if not allow_overwrite:
             if name in BACKENDS:
-                raise ValueError(f"Backend '{name}' is already registered as backend.")
+                raise ValueError(
+                    f"Backend '{name}' is already registered as backend.")
         BACKENDS[name] = backend
 
     @staticmethod
@@ -430,8 +444,9 @@ class BlueqatGlobalSetting:
     @staticmethod
     def remove_backend(name):
         """This method is deperecated. Use `unregister_backend` method."""
-        warnings.warn("remove_backend is deprecated. `unregister_backend` is recommended.",
-                      DeprecationWarning)
+        warnings.warn(
+            "remove_backend is deprecated. `unregister_backend` is recommended.",
+            DeprecationWarning)
         BlueqatGlobalSetting.unregister_backend(name)
 
     @staticmethod
@@ -456,3 +471,6 @@ class BlueqatGlobalSetting:
             str: The name of default backend.
         """
         return DEFAULT_BACKEND_NAME
+
+
+from .backends import BACKENDS, DEFAULT_BACKEND_NAME
