@@ -4,15 +4,16 @@ This module is internally used.
 """
 
 import math
-from abc import ABC
-from typing import List
+from typing import Callable, List, Optional
+
+import numpy as np
 
 
-class Gate(ABC):
-    """Abstract quantum gate class."""
+class Operation:
+    """Abstract quantum circuit operation class."""
 
-    """Lower name of the gate."""
-    lowername: str = None
+    """Lower name of the operation."""
+    lowername: Optional[str] = None
 
     @property
     def uppername(self) -> str:
@@ -21,19 +22,27 @@ class Gate(ABC):
 
     def __init__(self, targets, params=(), **kwargs):
         if self.lowername is None:
-            raise ValueError(f"{self.__class__.__name__}.lowername is not defined.")
+            raise ValueError(
+                f"{self.__class__.__name__}.lowername is not defined.")
         self.params = params
         self.kwargs = kwargs
         self.targets = targets
 
-    def fallback(self, n_qubits: int) -> List['Gate']:
-        """Returns alternative gates to make equivalent circuit."""
-        raise NotImplementedError(f"The fallback of {self.__class__.__name__} gate is not defined.")
+    def target_iter(self, n_qubits):
+        """The generator which yields the target qubits."""
+        return slicing(self.targets, n_qubits)
 
-    def dagger(self) -> 'Gate':
+    def fallback(self, n_qubits: int) -> List['Operation']:
+        """Returns alternative operations to make equivalent circuit."""
+        raise NotImplementedError(
+            f"The fallback of {self.__class__.__name__} operation is not defined."
+        )
+
+    def dagger(self) -> 'Operation':
         """Returns the Hermitian conjugate of `self`."""
-        raise NotImplementedError(f"Hermitian conjugate of this gate is not provided.")
-
+        raise NotImplementedError(
+            f"Hermitian conjugate of {self.__class__.__name__} operation is not provided."
+        )
 
     def _str_args(self) -> str:
         """Returns printable string of args."""
@@ -66,12 +75,37 @@ class Gate(ABC):
         return f'{self.lowername}{str_args}{str_targets}'
 
 
+class Gate(Operation):
+    """Abstract quantum gate class."""
+    @property
+    def n_qargs(self) -> int:
+        """Number of qubit arguments of this gate."""
+        raise NotImplementedError()
+
+    def fallback(self, n_qubits: int) -> List['Gate']:
+        """Returns alternative gates to make equivalent circuit."""
+        raise NotImplementedError(
+            f"The fallback of {self.__class__.__name__} gate is not defined.")
+
+    def dagger(self) -> 'Gate':
+        """Returns the Hermitian conjugate of `self`."""
+        raise NotImplementedError(
+            f"Hermitian conjugate of this gate is not provided.")
+
+    def matrix(self) -> np.ndarray:
+        """Returns the matrix of implementations.
+
+        (Non-abstract) subclasses of Gate must implement this method.
+        WARNING: qubit order specifications of multi qubit gate is still not defined.
+        """
+        raise NotImplementedError()
+
+
 class OneQubitGate(Gate):
     """Abstract quantum gate class for 1 qubit gate."""
-
-    def target_iter(self, n_qubits):
-        """The generator which yields the target qubits."""
-        return slicing(self.targets, n_qubits)
+    @property
+    def n_qargs(self):
+        return 1
 
     def _make_fallback_for_target_iter(self, n_qubits, fallback):
         gates = []
@@ -82,6 +116,9 @@ class OneQubitGate(Gate):
 
 class TwoQubitGate(Gate):
     """Abstract quantum gate class for 2 qubits gate."""
+    @property
+    def n_qargs(self):
+        return 2
 
     def control_target_iter(self, n_qubits):
         """The generator which yields the tuples of (control, target) qubits."""
@@ -104,6 +141,9 @@ class IGate(OneQubitGate):
     def dagger(self):
         return self
 
+    def matrix(self):
+        return np.eye(2)
+
 
 class XGate(OneQubitGate):
     """Pauli's X gate"""
@@ -112,6 +152,9 @@ class XGate(OneQubitGate):
     def dagger(self):
         return self
 
+    def matrix(self):
+        return np.array([[0, 1], [1, 0]], dtype=complex)
+
 
 class YGate(OneQubitGate):
     """Pauli's Y gate"""
@@ -120,6 +163,9 @@ class YGate(OneQubitGate):
     def dagger(self):
         return self
 
+    def matrix(self):
+        return np.array([[0, -1j], [1j, 0]])
+
 
 class ZGate(OneQubitGate):
     """Pauli's Z gate"""
@@ -128,6 +174,9 @@ class ZGate(OneQubitGate):
     def dagger(self):
         return self
 
+    def matrix(self):
+        return np.array([[1, 0], [0, -1]], dtype=complex)
+
 
 class HGate(OneQubitGate):
     """Hadamard gate"""
@@ -136,42 +185,61 @@ class HGate(OneQubitGate):
     def dagger(self):
         return self
 
+    def matrix(self):
+        return np.array([[1, 1], [1, -1]], dtype=complex) / np.sqrt(2)
+
 
 class RXGate(OneQubitGate):
     """Rotate-X gate"""
     lowername = "rx"
 
     def __init__(self, targets, theta, **kwargs):
-        super().__init__(targets, (theta,), **kwargs)
+        super().__init__(targets, (theta, ), **kwargs)
         self.theta = theta
 
     def dagger(self):
         return RXGate(self.targets, -self.theta, **self.kwargs)
 
+    def matrix(self):
+        t = self.theta * 0.5
+        a = np.cos(t)
+        b = -1j * np.sin(t)
+        return np.array([[a, b], [b, a]], dtype=complex)
+
 
 class RYGate(OneQubitGate):
     """Rotate-Y gate"""
     lowername = "ry"
 
     def __init__(self, targets, theta, **kwargs):
-        super().__init__(targets, (theta,), **kwargs)
+        super().__init__(targets, (theta, ), **kwargs)
         self.theta = theta
 
     def dagger(self):
         return RYGate(self.targets, -self.theta, **self.kwargs)
 
+    def matrix(self):
+        t = self.theta * 0.5
+        a = np.cos(t)
+        b = np.sin(t)
+        return np.array([[a, -b], [b, a]], dtype=complex)
+
 
 class RZGate(OneQubitGate):
     """Rotate-Z gate"""
     lowername = "rz"
 
     def __init__(self, targets, theta, **kwargs):
-        super().__init__(targets, (theta,), **kwargs)
+        super().__init__(targets, (theta, ), **kwargs)
         self.theta = theta
 
     def dagger(self):
         return RZGate(self.targets, -self.theta, **self.kwargs)
 
+    def matrix(self):
+        a = np.exp(0.5j * self.theta)
+        return np.array([[a.conjugate(), 0], [0, a]], dtype=complex)
+
 
 class PhaseGate(OneQubitGate):
     """Rotate-Z gate but global phase is different.
@@ -186,7 +254,7 @@ class PhaseGate(OneQubitGate):
     lowername = "phase"
 
     def __init__(self, targets, theta, **kwargs):
-        super().__init__(targets, (theta,), **kwargs)
+        super().__init__(targets, (theta, ), **kwargs)
         self.theta = theta
 
     def dagger(self):
@@ -194,7 +262,11 @@ class PhaseGate(OneQubitGate):
 
     def fallback(self, n_qubits):
         # If phase gate is not implemented in the backend, global phase is ignored.
-        return self._make_fallback_for_target_iter(n_qubits, lambda t: [RZGate(t, self.theta)])
+        return self._make_fallback_for_target_iter(
+            n_qubits, lambda t: [RZGate(t, self.theta)])
+
+    def matrix(self):
+        return np.array([[1, 0], [0, np.exp(self.theta)]], dtype=complex)
 
 
 class TGate(OneQubitGate):
@@ -205,7 +277,11 @@ class TGate(OneQubitGate):
         return TDagGate(self.targets, self.params, **self.kwargs)
 
     def fallback(self, n_qubits):
-        return self._make_fallback_for_target_iter(n_qubits, lambda t: [PhaseGate(t, math.pi / 4)])
+        return self._make_fallback_for_target_iter(
+            n_qubits, lambda t: [PhaseGate(t, math.pi / 4)])
+
+    def matrix(self):
+        return np.array([[1, 0], [0, np.exp(np.pi * 0.25)]])
 
 
 class TDagGate(OneQubitGate):
@@ -216,7 +292,11 @@ class TDagGate(OneQubitGate):
         return TGate(self.targets, self.params, **self.kwargs)
 
     def fallback(self, n_qubits):
-        return self._make_fallback_for_target_iter(n_qubits, lambda t: [PhaseGate(t, -math.pi / 4)])
+        return self._make_fallback_for_target_iter(
+            n_qubits, lambda t: [PhaseGate(t, -math.pi / 4)])
+
+    def matrix(self):
+        return np.array([[1, 0], [0, np.exp(np.pi * -0.25)]])
 
 
 class SGate(OneQubitGate):
@@ -227,7 +307,11 @@ class SGate(OneQubitGate):
         return SDagGate(self.targets, self.params, **self.kwargs)
 
     def fallback(self, n_qubits):
-        return self._make_fallback_for_target_iter(n_qubits, lambda t: [PhaseGate(t, math.pi / 2)])
+        return self._make_fallback_for_target_iter(
+            n_qubits, lambda t: [PhaseGate(t, math.pi / 2)])
+
+    def matrix(self):
+        return np.array([[1, 0], [0, 1j]])
 
 
 class SDagGate(OneQubitGate):
@@ -238,7 +322,11 @@ class SDagGate(OneQubitGate):
         return SGate(self.targets, self.params, **self.kwargs)
 
     def fallback(self, n_qubits):
-        return self._make_fallback_for_target_iter(n_qubits, lambda t: [PhaseGate(t, -math.pi / 2)])
+        return self._make_fallback_for_target_iter(
+            n_qubits, lambda t: [PhaseGate(t, -math.pi / 2)])
+
+    def matrix(self):
+        return np.array([[1, 0], [0, -1j]])
 
 
 class U1Gate(OneQubitGate):
@@ -252,7 +340,7 @@ class U1Gate(OneQubitGate):
     lowername = "u1"
 
     def __init__(self, targets, lambd, **kwargs):
-        super().__init__(targets, (lambd,), **kwargs)
+        super().__init__(targets, (lambd, ), **kwargs)
         self.lambd = lambd
 
     def dagger(self):
@@ -262,6 +350,10 @@ class U1Gate(OneQubitGate):
         return self._make_fallback_for_target_iter(
             n_qubits, lambda t: [U3Gate(t, 0.0, 0.0, self.lambd)])
 
+    def matrix(self):
+        a = np.exp(0.5j * self.theta)
+        return np.array([[a.conjugate(), 0], [0, a]], dtype=complex)
+
 
 class U2Gate(OneQubitGate):
     """U2 gate"""
@@ -273,12 +365,21 @@ class U2Gate(OneQubitGate):
         self.lambd = lambd
 
     def dagger(self):
-        return U3Gate(self.targets, -math.pi / 2, -self.lambd, -self.phi, **self.kwargs)
+        return U3Gate(self.targets, -math.pi / 2, -self.lambd, -self.phi,
+                      **self.kwargs)
 
     def fallback(self, n_qubits):
         return self._make_fallback_for_target_iter(
             n_qubits, lambda t: [U3Gate(t, math.pi / 2, self.phi, self.lambd)])
 
+    def matrix(self):
+        p, l = self.params
+        c = 1.0 / np.sqrt(2)
+        a = np.exp(0.5j * (p + l)) * c
+        b = np.exp(0.5j * (p - l)) * c
+        return np.array([[a.conjugate(), -b.conjugate()], [a, b]],
+                        dtype=complex)
+
 
 class U3Gate(OneQubitGate):
     """U3 gate"""
@@ -291,7 +392,34 @@ class U3Gate(OneQubitGate):
         self.lambd = lambd
 
     def dagger(self):
-        return U3Gate(self.targets, -self.theta, -self.lambd, -self.phi, **self.kwargs)
+        return U3Gate(self.targets, -self.theta, -self.lambd, -self.phi,
+                      **self.kwargs)
+
+    def matrix(self):
+        t, p, l = self.params
+        a = np.exp(0.5j * (p + l)) * np.cos(t * 0.5)
+        b = np.exp(0.5j * (p - l)) * np.sin(t * 0.5)
+        return np.array([[a.conjugate(), -b.conjugate()], [b, a]],
+                        dtype=complex)
+
+
+class Mat1Gate(OneQubitGate):
+    """Arbitrary 2x2 matrix gate
+
+    `mat` is expected a 2x2 unitary matrix, but not checked.
+    (If unexpected matrix is given, backend may raises error or returns weird result)
+    """
+    lowername = "mat1"
+
+    def __init__(self, targets, mat: np.ndarray, **kwargs):
+        super().__init__(targets, (mat, ), **kwargs)
+        self.mat = mat
+
+    def dagger(self):
+        return Mat1Gate(self.targets, self.mat.T.conjugate(), **self.kwargs)
+
+    def matrix(self):
+        return self.mat
 
 
 class CXGate(TwoQubitGate):
@@ -301,6 +429,12 @@ class CXGate(TwoQubitGate):
     def dagger(self):
         return self
 
+    # TODO: Specification required: target \otimes control or control \otimes target
+    def matrix(self):
+        return np.array(
+            [[1, 0, 0, 0], [0, 0, 0, 1], [0, 0, 1, 0], [0, 1, 0, 0]],
+            dtype=complex)
+
 
 class CZGate(TwoQubitGate):
     """Controlled-Z gate"""
@@ -309,6 +443,11 @@ class CZGate(TwoQubitGate):
     def dagger(self):
         return self
 
+    def matrix(self):
+        return np.array(
+            [[1, 0, 0, 0], [0, 1, 0, 0], [0, 0, 1, 0], [0, 0, 0, -1]],
+            dtype=complex)
+
 
 class CYGate(TwoQubitGate):
     """Controlled-Y gate"""
@@ -320,9 +459,13 @@ class CYGate(TwoQubitGate):
     def fallback(self, n_qubits):
         return self._make_fallback_for_control_target_iter(
             n_qubits,
-            lambda c, t: [SDagGate(t),
-                          CXGate((c, t)),
-                          SGate(t)])
+            lambda c, t: [SDagGate(t), CXGate(
+                (c, t)), SGate(t)])
+
+    def matrix(self):
+        return np.array(
+            [[1, 0, 0, 0], [0, 0, -1j, 0], [0, 0, 1, 0], [0, 1j, 0, 0]],
+            dtype=complex)
 
 
 class CHGate(TwoQubitGate):
@@ -335,10 +478,16 @@ class CHGate(TwoQubitGate):
     def fallback(self, n_qubits):
         # Ignores global phase
         return self._make_fallback_for_control_target_iter(
-            n_qubits,
-            lambda c, t: [RYGate(t, math.pi / 4),
-                          CXGate((c, t)),
-                          RYGate(t, -math.pi / 4)])
+            n_qubits, lambda c, t:
+            [RYGate(t, math.pi / 4),
+             CXGate((c, t)),
+             RYGate(t, -math.pi / 4)])
+
+    def matrix(self):
+        a = 1.0 / np.sqrt(2)
+        return np.array(
+            [[1, 0, 0, 0], [0, a, 0, a], [0, 0, 1, 0], [0, a, 0, -a]],
+            dtype=complex)
 
 
 class CRXGate(TwoQubitGate):
@@ -346,7 +495,7 @@ class CRXGate(TwoQubitGate):
     lowername = "crx"
 
     def __init__(self, targets, theta, **kwargs):
-        super().__init__(targets, (theta,), **kwargs)
+        super().__init__(targets, (theta, ), **kwargs)
         self.theta = theta
 
     def dagger(self):
@@ -354,11 +503,20 @@ class CRXGate(TwoQubitGate):
 
     def fallback(self, n_qubits):
         return self._make_fallback_for_control_target_iter(
-            n_qubits,
-            lambda c, t: [RXGate(t, self.theta / 2),
-                          CZGate((c, t)),
-                          RXGate(t, -self.theta / 2),
-                          CZGate((c, t))])
+            n_qubits, lambda c, t: [
+                RXGate(t, self.theta / 2),
+                CZGate((c, t)),
+                RXGate(t, -self.theta / 2),
+                CZGate((c, t))
+            ])
+
+    def matrix(self):
+        t = self.theta * 0.5
+        a = np.cos(t)
+        b = -1j * np.sin(t)
+        return np.array(
+            [[1, 0, 0, 0], [0, a, 0, b], [0, 0, 1, 0], [0, b, 0, a]],
+            dtype=complex)
 
 
 class CRYGate(TwoQubitGate):
@@ -366,7 +524,7 @@ class CRYGate(TwoQubitGate):
     lowername = "cry"
 
     def __init__(self, targets, theta, **kwargs):
-        super().__init__(targets, (theta,), **kwargs)
+        super().__init__(targets, (theta, ), **kwargs)
         self.theta = theta
 
     def dagger(self):
@@ -374,11 +532,20 @@ class CRYGate(TwoQubitGate):
 
     def fallback(self, n_qubits):
         return self._make_fallback_for_control_target_iter(
-            n_qubits,
-            lambda c, t: [RYGate(t, self.theta / 2),
-                          CXGate((c, t)),
-                          RYGate(t, -self.theta / 2),
-                          CXGate((c, t))])
+            n_qubits, lambda c, t: [
+                RYGate(t, self.theta / 2),
+                CXGate((c, t)),
+                RYGate(t, -self.theta / 2),
+                CXGate((c, t))
+            ])
+
+    def matrix(self):
+        t = self.theta * 0.5
+        a = np.cos(t)
+        b = np.sin(t)
+        return np.array(
+            [[1, 0, 0, 0], [0, a, 0, -b], [0, 0, 1, 0], [0, b, 0, a]],
+            dtype=complex)
 
 
 class CRZGate(TwoQubitGate):
@@ -386,7 +553,7 @@ class CRZGate(TwoQubitGate):
     lowername = "crz"
 
     def __init__(self, targets, theta, **kwargs):
-        super().__init__(targets, (theta,), **kwargs)
+        super().__init__(targets, (theta, ), **kwargs)
         self.theta = theta
 
     def dagger(self):
@@ -394,11 +561,18 @@ class CRZGate(TwoQubitGate):
 
     def fallback(self, n_qubits):
         return self._make_fallback_for_control_target_iter(
-            n_qubits,
-            lambda c, t: [RZGate(t, self.theta / 2),
-                          CXGate((c, t)),
-                          RZGate(t, -self.theta / 2),
-                          CXGate((c, t))])
+            n_qubits, lambda c, t: [
+                RZGate(t, self.theta / 2),
+                CXGate((c, t)),
+                RZGate(t, -self.theta / 2),
+                CXGate((c, t))
+            ])
+
+    def matrix(self):
+        a = np.exp(0.5j * self.theta)
+        return np.array([[1, 0, 0, 0], [0, a.conjugate(), 0, 0], [0, 0, 1, 0],
+                         [0, 0, 0, a]],
+                        dtype=complex)
 
 
 class CPhaseGate(TwoQubitGate):
@@ -406,7 +580,7 @@ class CPhaseGate(TwoQubitGate):
     lowername = "cphase"
 
     def __init__(self, targets, theta, **kwargs):
-        super().__init__(targets, (theta,), **kwargs)
+        super().__init__(targets, (theta, ), **kwargs)
         self.theta = theta
 
     def dagger(self):
@@ -414,9 +588,14 @@ class CPhaseGate(TwoQubitGate):
 
     def fallback(self, n_qubits):
         return self._make_fallback_for_control_target_iter(
-            n_qubits,
-            lambda c, t: [CRZGate((c, t), self.theta),
-                          PhaseGate(c, self.theta / 2)])
+            n_qubits, lambda c, t:
+            [CRZGate((c, t), self.theta),
+             PhaseGate(c, self.theta / 2)])
+
+    def matrix(self):
+        return np.array([[1, 0, 0, 0], [0, 1, 0, 0], [0, 0, 1, 0],
+                         [0, 0, 0, 0, np.exp(self.theta)]],
+                        dtype=complex)
 
 
 class RXXGate(TwoQubitGate):
@@ -424,7 +603,7 @@ class RXXGate(TwoQubitGate):
     lowername = "rxx"
 
     def __init__(self, targets, theta, **kwargs):
-        super().__init__(targets, (theta,), **kwargs)
+        super().__init__(targets, (theta, ), **kwargs)
         self.theta = theta
 
     def dagger(self):
@@ -433,12 +612,20 @@ class RXXGate(TwoQubitGate):
     def fallback(self, n_qubits):
         # TODO: test
         return self._make_fallback_for_control_target_iter(
-            n_qubits,
-            lambda c, t: [HGate(c),
-                          HGate(t),
-                          RZZGate((c, t), self.theta),
-                          HGate(c),
-                          HGate(t)])
+            n_qubits, lambda c, t: [
+                HGate(c),
+                HGate(t),
+                RZZGate((c, t), self.theta),
+                HGate(c),
+                HGate(t)
+            ])
+
+    def matrix(self):
+        a = np.cos(self.theta * 0.5)
+        b = -1j * np.sin(self.theta * 0.5)
+        return np.array(
+            [[a, 0, 0, b], [0, a, b, 0], [0, b, a, 0], [a, 0, 0, b]],
+            dtype=complex)
 
 
 class RYYGate(TwoQubitGate):
@@ -446,7 +633,7 @@ class RYYGate(TwoQubitGate):
     lowername = "ryy"
 
     def __init__(self, targets, theta, **kwargs):
-        super().__init__(targets, (theta,), **kwargs)
+        super().__init__(targets, (theta, ), **kwargs)
         self.theta = theta
 
     def dagger(self):
@@ -455,12 +642,20 @@ class RYYGate(TwoQubitGate):
     def fallback(self, n_qubits):
         # TODO: test
         return self._make_fallback_for_control_target_iter(
-            n_qubits,
-            lambda c, t: [RXGate(c, -math.pi * 0.5),
-                          RXGate(t, -math.pi * 0.5),
-                          RZZGate((c, t), self.theta),
-                          RXGate(c, math.pi * 0.5),
-                          RXGate(t, math.pi * 0.5)])
+            n_qubits, lambda c, t: [
+                RXGate(c, -math.pi * 0.5),
+                RXGate(t, -math.pi * 0.5),
+                RZZGate((c, t), self.theta),
+                RXGate(c, math.pi * 0.5),
+                RXGate(t, math.pi * 0.5)
+            ])
+
+    def matrix(self):
+        a = np.cos(self.theta * 0.5)
+        b = 1j * np.sin(self.theta * 0.5)
+        return np.array(
+            [[a, 0, 0, b], [0, a, -b, 0], [0, -b, a, 0], [a, 0, 0, b]],
+            dtype=complex)
 
 
 class RZZGate(TwoQubitGate):
@@ -468,7 +663,7 @@ class RZZGate(TwoQubitGate):
     lowername = "rzz"
 
     def __init__(self, targets, theta, **kwargs):
-        super().__init__(targets, (theta,), **kwargs)
+        super().__init__(targets, (theta, ), **kwargs)
         self.theta = theta
 
     def dagger(self):
@@ -477,10 +672,16 @@ class RZZGate(TwoQubitGate):
     def fallback(self, n_qubits):
         # TODO: test
         return self._make_fallback_for_control_target_iter(
-            n_qubits,
-            lambda c, t: [CXGate((c, t)),
-                          RZGate(t, self.theta),
-                          CXGate((c, t))])
+            n_qubits, lambda c, t:
+            [CXGate(
+                (c, t)), RZGate(t, self.theta),
+             CXGate((c, t))])
+
+    def matrix(self):
+        a = np.exp(0.5j * self.theta)
+        return np.array([[a.conjugate(), 0, 0, 0], [0, a, 0, 0], [0, 0, a, 0],
+                         [0, 0, 0, a.conjugate()]],
+                        dtype=complex)
 
 
 class SwapGate(TwoQubitGate):
@@ -493,7 +694,41 @@ class SwapGate(TwoQubitGate):
     def fallback(self, n_qubits):
         return self._make_fallback_for_control_target_iter(
             n_qubits,
-            lambda c, t: [CXGate((c, t)), CXGate((t, c)), CXGate((c, t))])
+            lambda c, t: [CXGate(
+                (c, t)), CXGate(
+                    (t, c)), CXGate((c, t))])
+
+    def matrix(self):
+        return np.array([[1, 0, 0, 0], [0, 0, 1, 0], [0, 1, 0, 0],
+                         [0, 0, 0, 1]])
+
+
+class ZZGate(TwoQubitGate):
+    """ZZ gate
+
+    This gate is a basis two-qubit gate for some kinds of trapped-ion based machines.
+    It is equivalent with RZZ(pi/2) except global phase.
+    """
+    lowername = "zz"
+
+    def __init__(self, targets, **kwargs):
+        super().__init__(targets, (), **kwargs)
+
+    def dagger(self):
+        return self
+
+    def fallback(self, n_qubits):
+        # Ignoring global phase.
+        return self._make_fallback_for_control_target_iter(
+            n_qubits, lambda c, t: [
+                RYGate(t, math.pi * 0.5),
+                CXGate((c, t)),
+                RZGate(c, math.pi * 0.5),
+                U3Gate(t, -math.pi * 0.5, math.pi * 0.5, 0),
+            ])
+
+        def matrix(self):
+            return np.diag([1, 1j, 1j, 1])
 
 
 class CU1Gate(TwoQubitGate):
@@ -507,7 +742,7 @@ class CU1Gate(TwoQubitGate):
     lowername = "cu1"
 
     def __init__(self, targets, lambd, **kwargs):
-        super().__init__(targets, (lambd,), **kwargs)
+        super().__init__(targets, (lambd, ), **kwargs)
         self.lambd = lambd
 
     def dagger(self):
@@ -523,6 +758,11 @@ class CU1Gate(TwoQubitGate):
                 U1Gate(t, self.lambd / 2),
             ])
 
+    def matrix(self):
+        return np.array([[1, 0, 0, 0], [0, 1, 0, 0], [0, 0, 1, 0],
+                         [0, 0, 0, 0, np.exp(self.theta)]],
+                        dtype=complex)
+
 
 class CU2Gate(TwoQubitGate):
     """Controlled U2 gate"""
@@ -534,11 +774,22 @@ class CU2Gate(TwoQubitGate):
         self.lambd = lambd
 
     def dagger(self):
-        return CU3Gate(self.targets, -math.pi / 2, -self.lambd, -self.phi, **self.kwargs)
+        return CU3Gate(self.targets, -math.pi / 2, -self.lambd, -self.phi,
+                       **self.kwargs)
 
     def fallback(self, n_qubits):
         return self._make_fallback_for_control_target_iter(
-            n_qubits, lambda c, t: [CU3Gate((c, t), math.pi / 2, self.phi, self.lambd)])
+            n_qubits,
+            lambda c, t: [CU3Gate((c, t), math.pi / 2, self.phi, self.lambd)])
+
+    def matrix(self):
+        p, l = self.params
+        c = 1 / np.sqrt(2)
+        a = np.exp(0.5j * (p + l)) * c
+        b = np.exp(0.5j * (p - l)) * c
+        return np.array([[1, 0, 0, 0], [0, a.conjugate(), 0, -b.conjugate()],
+                         [0, 0, 1, 0], [0, a, 0, b]],
+                        dtype=complex)
 
 
 class CU3Gate(TwoQubitGate):
@@ -552,7 +803,8 @@ class CU3Gate(TwoQubitGate):
         self.lambd = lambd
 
     def dagger(self):
-        return CU3Gate(self.targets, -self.theta, -self.lambd, -self.phi, **self.kwargs)
+        return CU3Gate(self.targets, -self.theta, -self.lambd, -self.phi,
+                       **self.kwargs)
 
     def fallback(self, n_qubits):
         return self._make_fallback_for_control_target_iter(
@@ -564,11 +816,23 @@ class CU3Gate(TwoQubitGate):
                 U3Gate(t, self.theta / 2, self.phi, 0),
             ])
 
+    def matrix(self):
+        t, p, l = self.params
+        a = np.exp(0.5j * (p + l)) * np.cos(t * 0.5)
+        b = np.exp(0.5j * (p - l)) * np.sin(t * 0.5)
+        return np.array([[1, 0, 0, 0], [0, a.conjugate(), 0, -b.conjugate()],
+                         [0, 0, 1, 0], [0, a, 0, b]],
+                        dtype=complex)
+
 
 class ToffoliGate(Gate):
     """Toffoli (CCX) gate"""
     lowername = "ccx"
 
+    @property
+    def n_qargs(self):
+        return 3
+
     def dagger(self):
         return self
 
@@ -576,6 +840,29 @@ class ToffoliGate(Gate):
         c1, c2, t = self.targets
         return [
             HGate(t),
+            CCZGate((c1, c2, t)),
+            HGate(t),
+        ]
+
+    def matrix(self):
+        return np.array([[1, 0, 0, 0, 0, 0, 0, 0], [0, 1, 0, 0, 0, 0, 0, 0],
+                         [0, 0, 1, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 1],
+                         [0, 0, 0, 0, 1, 0, 0, 0], [0, 0, 0, 0, 0, 1, 0, 0],
+                         [0, 0, 0, 0, 0, 0, 1, 0], [0, 0, 0, 1, 0, 0, 0, 0]],
+                        dtype=complex)
+
+
+class CCZGate(Gate):
+    """2-Controlled Z gate"""
+    lowername = "ccz"
+
+    @property
+    def n_qargs(self):
+        return 3
+
+    def fallback(self, n_qubits):
+        c1, c2, t = self.targets
+        return [
             CXGate((c2, t)),
             TDagGate(t),
             CXGate((c1, t)),
@@ -585,51 +872,65 @@ class ToffoliGate(Gate):
             CXGate((c1, t)),
             TGate(c2),
             TGate(t),
-            HGate(t),
             CXGate((c1, c2)),
             TGate(c1),
             TDagGate(c2),
             CXGate((c1, c2)),
         ]
 
-
-class CCZGate(Gate):
-    """2-Controlled Z gate"""
-    lowername = "ccz"
-
     def dagger(self):
         return self
 
+    def matrix(self):
+        return np.array([[1, 0, 0, 0, 0, 0, 0, 0], [0, 1, 0, 0, 0, 0, 0, 0],
+                         [0, 0, 1, 0, 0, 0, 0, 0], [0, 0, 0, 1, 0, 0, 0, 0],
+                         [0, 0, 0, 0, 1, 0, 0, 0], [0, 0, 0, 0, 0, 1, 0, 0],
+                         [0, 0, 0, 0, 0, 0, 1, 0], [0, 0, 0, 0, 0, 0, 0, -1]],
+                        dtype=complex)
+
 
 class CSwapGate(Gate):
     """Controlled SWAP gate"""
     lowername = "cswap"
 
+    @property
+    def n_qargs(self):
+        return 3
+
     def dagger(self):
         return self
 
     def fallback(self, n_qubits):
         # TODO: test
         c, t1, t2 = self.targets
-        return [CXGate((t2, t1)),
-                ToffoliGate((c, t1, t2)),
-                CXGate((t2, t1))]
+        return [CXGate((t2, t1)), ToffoliGate((c, t1, t2)), CXGate((t2, t1))]
+
+    def matrix(self):
+        return np.array([[1, 0, 0, 0, 0, 0, 0, 0], [0, 1, 0, 0, 0, 0, 0, 0],
+                         [0, 0, 1, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 1, 0, 0],
+                         [0, 0, 0, 0, 1, 0, 0, 0], [0, 0, 0, 1, 0, 0, 0, 0],
+                         [0, 0, 0, 0, 0, 0, 1, 0], [0, 0, 0, 0, 0, 0, 0, 1]],
+                        dtype=complex)
 
 
-class Measurement(OneQubitGate):
+class Measurement(Operation):
     """Measurement operation"""
     lowername = "measure"
 
-    def dagger(self):
-        raise ValueError(f"Hermitian conjugate of the measurement is not available.")
+    # Ad-hoc copy and paste programming.
+    def target_iter(self, n_qubits):
+        """The generator which yields the target qubits."""
+        return slicing(self.targets, n_qubits)
 
 
-class Reset(OneQubitGate):
+class Reset(Operation):
     """Reset operation"""
     lowername = "reset"
 
-    def dagger(self):
-        raise ValueError(f"Hermitian conjugate of the reset operation is not available.")
+    # Ad-hoc copy and paste programming.
+    def target_iter(self, n_qubits):
+        """The generator which yields the target qubits."""
+        return slicing(self.targets, n_qubits)
 
 
 def slicing_singlevalue(arg, length):
@@ -649,7 +950,8 @@ def slicing_singlevalue(arg, length):
         try:
             i = arg.__index__()
         except AttributeError:
-            raise TypeError("indices must be integers or slices, not " + arg.__class__.__name__)
+            raise TypeError("indices must be integers or slices, not " +
+                            arg.__class__.__name__)
         if i < 0:
             i += length
         yield i
@@ -673,10 +975,12 @@ def qubit_pairs(args, length):
     controls = list(slicing(args[0], length))
     targets = list(slicing(args[1], length))
     if len(controls) != len(targets):
-        raise ValueError("The number of control qubits and target qubits are must be same.")
+        raise ValueError(
+            "The number of control qubits and target qubits are must be same.")
     for c, z in zip(controls, targets):
         if c == z:
-            raise ValueError("Control qubit and target qubit are must be different.")
+            raise ValueError(
+                "Control qubit and target qubit are must be different.")
     return zip(controls, targets)
 
 
@@ -693,6 +997,7 @@ def get_maximum_index(indices):
             return max(start, stop - 1)
         else:
             return idx.__index__()
+
     if isinstance(indices, tuple):
         return max((_maximum_idx_single(i) for i in indices), default=-1)
     else:
